@@ -1,47 +1,27 @@
 #include "evas_common.h"
 #include "evas_private.h"
 
-/* FIXME: this broken e17's mouse cursor - need to figure out why */
-
-/* uncomment the next line if smart objects should be informed
- * if they are moved to the position they are already in
- * (e.g. if they are in 0,0 and you call evas_object_move(o, 0, 0)
- */
-//#define FORWARD_NOOP_MOVES_TO_SMART_OBJS
-
-/* likewise, for resizes
- */
-//#define FORWARD_NOOP_RESIZES_TO_SMART_OBJS
-
-/* Similar to FORWARD_NOOP_*, this will allow no-operations (those calls that
- * have values exactly like current state) to call EVAS_CALLBACK_*.
- *
- * For some unknown reason this was the default behavior so it is left as
- * a compile-time option, but will be deprecated and removed soon as it can
- * potentially lead to execution of unnecessary code.
- * by Gustavo, February 5th, 2009.
- * XXX: remove-me before e17 release!
- */
-//#define CALLBACK_NOOP
-
+EVAS_MEMPOOL(_mp_obj);
+EVAS_MEMPOOL(_mp_sh);
 
 static Eina_Inlist *
 get_layer_objects(Evas_Layer *l)
 {
-   if( !l || !l->objects ) return NULL;
-
+   if ((!l) || (!l->objects)) return NULL;
    return (EINA_INLIST_GET(l->objects));
 }
 
 /* evas internal stuff */
 Evas_Object *
-evas_object_new(Evas *e)
+evas_object_new(Evas *e __UNUSED__)
 {
    Evas_Object *obj;
 
-   obj = calloc(1, sizeof(Evas_Object));
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object", Evas_Object, 512, NULL);
+   obj = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object);
    if (!obj) return NULL;
-
+   EVAS_MEMPOOL_PREP(_mp_obj, obj, Evas_Object);
+  
    obj->magic = MAGIC_OBJ;
    obj->cur.scale = 1.0;
    obj->prev.scale = 1.0;
@@ -75,8 +55,11 @@ evas_object_free(Evas_Object *obj, int clean_layer)
 	free(node);
      }
    obj->magic = 0;
-   if (obj->size_hints) free(obj->size_hints);
-   free(obj);
+   if (obj->size_hints)
+     {
+       EVAS_MEMPOOL_FREE(_mp_sh, obj->size_hints);
+     }
+   EVAS_MEMPOOL_FREE(_mp_obj, obj);
 }
 
 void
@@ -84,13 +67,36 @@ evas_object_change(Evas_Object *obj)
 {
    Eina_List *l;
    Evas_Object *obj2;
-
+   Eina_Bool movch = 0;
+   
+   if (obj->layer->evas->nochange)
+     {
+//        printf("nochange %p\n", obj);
+        return;
+     }
+//   else
+//      printf("ch %p\n", obj);
    obj->layer->evas->changed = 1;
-   if (obj->changed) return;
+   if (obj->changed_move)
+     {
+        movch = 1;
+        obj->changed_move = 0;
+        if (!obj->changed_nomove) obj->changed_move_only = 1;
+        if (obj->changed) return;
+     }
+   else
+     {
+        obj->changed_move_only = 0;
+        obj->changed_nomove = 1;
+        if (obj->changed) return;
+     }
 //   obj->changed = 1;
    evas_render_object_recalc(obj);
    /* set changed flag on all objects this one clips too */
-   EINA_LIST_FOREACH(obj->clip.clipees, l, obj2) evas_object_change(obj2);
+   if (!((movch) && (obj->is_static_clip)))
+     {
+        EINA_LIST_FOREACH(obj->clip.clipees, l, obj2) evas_object_change(obj2);
+     }
    if (obj->smart.parent) evas_object_change(obj->smart.parent);
 }
 
@@ -372,16 +378,7 @@ evas_object_del(Evas_Object *obj)
    evas_common_frameq_flush();
 #endif
 
-   _evas_object_event_new();
-
-   evas_object_event_callback_call(obj, EVAS_CALLBACK_DEL, NULL);
-   _evas_post_event_callback_call(obj->layer->evas);
-   if (obj->name) evas_object_name_set(obj, NULL);
-   if (!obj->layer)
-     {
-	evas_object_free(obj, 1);
-	return;
-     }
+   evas_object_hide(obj);
    if (obj->focused)
      {
 	obj->focused = 0;
@@ -390,9 +387,18 @@ evas_object_del(Evas_Object *obj)
 	evas_object_event_callback_call(obj, EVAS_CALLBACK_FOCUS_OUT, NULL);
         _evas_post_event_callback_call(obj->layer->evas);
      }
+   _evas_object_event_new();
+   evas_object_event_callback_call(obj, EVAS_CALLBACK_DEL, NULL);
+   _evas_post_event_callback_call(obj->layer->evas);
+   if (obj->name) evas_object_name_set(obj, NULL);
+   if (!obj->layer)
+     {
+	evas_object_free(obj, 1);
+	return;
+     }
    obj->layer->evas->pointer.mouse_grabbed -= obj->mouse_grabbed;
    obj->mouse_grabbed = 0;
-   evas_object_hide(obj);
+   obj->mouse_in = 0;
    evas_object_grabs_cleanup(obj);
    while (obj->clip.clipees) evas_object_clip_unset(obj->clip.clipees->data);
    if (obj->cur.clipper) evas_object_clip_unset(obj);
@@ -423,28 +429,12 @@ evas_object_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
    if (evas_object_intercept_call_move(obj, x, y)) return;
-#ifdef FORWARD_NOOP_MOVES_TO_SMART_OBJS
-   if (obj->smart.smart)
+   if (obj->doing.in_move > 0)
      {
-       if (obj->smart.smart->smart_class->move)
-	  obj->smart.smart->smart_class->move(obj, x, y);
+        WRN("evas_object_move() called on object %p when in the middle of moving the same object", obj);
+        return;
      }
-#endif
-   if ((obj->cur.geometry.x == x) &&
-       (obj->cur.geometry.y == y))
-     {
-#ifdef CALLBACK_NOOP
-	evas_object_inform_call_move(obj);
-#endif
-	return;
-     }
-#ifndef FORWARD_NOOP_MOVES_TO_SMART_OBJS
-   if (obj->smart.smart)
-     {
-       if (obj->smart.smart->smart_class->move)
-	  obj->smart.smart->smart_class->move(obj, x, y);
-     }
-#endif
+   if ((obj->cur.geometry.x == x) && (obj->cur.geometry.y == y)) return;
    if (obj->layer->evas->events_frozen <= 0)
      {
 	pass = evas_event_passes_through(obj);
@@ -453,11 +443,19 @@ evas_object_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 					      obj->layer->evas->pointer.x,
 					      obj->layer->evas->pointer.y, 1, 1);
      }
+   obj->doing.in_move++;
+   if (obj->smart.smart)
+     {
+       if (obj->smart.smart->smart_class->move)
+	  obj->smart.smart->smart_class->move(obj, x, y);
+     }
    obj->cur.geometry.x = x;
    obj->cur.geometry.y = y;
 ////   obj->cur.cache.geometry.validity = 0;
+   obj->changed_move = 1;
    evas_object_change(obj);
    evas_object_clip_dirty(obj);
+   obj->doing.in_move--;
    if (obj->layer->evas->events_frozen <= 0)
      {
 	evas_object_recalc_clippees(obj);
@@ -500,7 +498,7 @@ evas_object_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
  *       @endcode
  *       This is more evident in images, but text, textblock, lines
  *       and polygons will behave similarly. Check their specific APIs
- *       to know how to achive your desired behavior.
+ *       to know how to achieve your desired behavior.
  *
  * @ingroup Evas_Object_Group_Basic
  */
@@ -515,28 +513,12 @@ evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
    if (obj->delete_me) return;
    if (w < 0) w = 0; if (h < 0) h = 0;
    if (evas_object_intercept_call_resize(obj, w, h)) return;
-#ifdef FORWARD_NOOP_RESIZES_TO_SMART_OBJS
-   if (obj->smart.smart)
+   if (obj->doing.in_resize > 0)
      {
-       if (obj->smart.smart->smart_class->resize)
-	  obj->smart.smart->smart_class->resize(obj, w, h);
+        WRN("evas_object_resize() called on object %p when in the middle of resizing the same object", obj);
+        return;
      }
-#endif
-   if ((obj->cur.geometry.w == w) &&
-       (obj->cur.geometry.h == h))
-     {
-#ifdef CALLBACK_NOOP
-	evas_object_inform_call_resize(obj);
-#endif
-	return;
-     }
-#ifndef FORWARD_NOOP_RESIZES_TO_SMART_OBJS
-   if (obj->smart.smart)
-     {
-       if (obj->smart.smart->smart_class->resize)
-	  obj->smart.smart->smart_class->resize(obj, w, h);
-     }
-#endif
+   if ((obj->cur.geometry.w == w) && (obj->cur.geometry.h == h)) return;
    if (obj->layer->evas->events_frozen <= 0)
      {
 	pass = evas_event_passes_through(obj);
@@ -545,12 +527,18 @@ evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
 					      obj->layer->evas->pointer.x,
 					      obj->layer->evas->pointer.y, 1, 1);
      }
-
+   obj->doing.in_resize++;
+   if (obj->smart.smart)
+     {
+       if (obj->smart.smart->smart_class->resize)
+	  obj->smart.smart->smart_class->resize(obj, w, h);
+     }
    obj->cur.geometry.w = w;
    obj->cur.geometry.h = h;
 ////   obj->cur.cache.geometry.validity = 0;
    evas_object_change(obj);
    evas_object_clip_dirty(obj);
+   obj->doing.in_resize--;
    /* NB: evas_object_recalc_clippees was here previously ( < 08/07/2009) */
    if (obj->layer->evas->events_frozen <= 0)
      {
@@ -622,7 +610,10 @@ _evas_object_size_hint_alloc(Evas_Object *obj)
 {
    if (obj->size_hints) return;
 
-   obj->size_hints = calloc(1, sizeof(Evas_Size_Hints));
+   EVAS_MEMPOOL_INIT(_mp_sh, "evas_size_hints", Evas_Size_Hints, 512, );
+   obj->size_hints = EVAS_MEMPOOL_ALLOC(_mp_sh, Evas_Size_Hints);
+   if (!obj->size_hints) return;
+   EVAS_MEMPOOL_PREP(_mp_sh, obj->size_hints, Evas_Size_Hints);
    obj->size_hints->max.w = -1;
    obj->size_hints->max.h = -1;
    obj->size_hints->align.x = 0.5;
@@ -1069,9 +1060,6 @@ evas_object_show(Evas_Object *obj)
      }
    if (obj->cur.visible)
      {
-#ifdef CALLBACK_NOOP
-	evas_object_inform_call_show(obj);
-#endif
 	return;
      }
    obj->cur.visible = 1;
@@ -1079,6 +1067,7 @@ evas_object_show(Evas_Object *obj)
    evas_object_clip_dirty(obj);
    if (obj->layer->evas->events_frozen <= 0)
      {
+        evas_object_clip_across_clippees_check(obj);
 	evas_object_recalc_clippees(obj);
 	if (!evas_event_passes_through(obj))
 	  {
@@ -1124,9 +1113,6 @@ evas_object_hide(Evas_Object *obj)
      }
    if (!obj->cur.visible)
      {
-#ifdef CALLBACK_NOOP
-	evas_object_inform_call_hide(obj);
-#endif
 	return;
      }
    obj->cur.visible = 0;
@@ -1134,10 +1120,12 @@ evas_object_hide(Evas_Object *obj)
    evas_object_clip_dirty(obj);
    if (obj->layer->evas->events_frozen <= 0)
      {
+        evas_object_clip_across_clippees_check(obj);
 	evas_object_recalc_clippees(obj);
 	if (!evas_event_passes_through(obj))
 	  {
-	     if (!obj->smart.smart)
+	     if ((!obj->smart.smart) ||
+                 ((obj->cur.map) && (obj->cur.map->count == 4) && (obj->cur.usemap)))
 	       {
 		  if (evas_object_is_in_output_rect(obj,
 						    obj->layer->evas->pointer.x,
@@ -1246,7 +1234,7 @@ evas_object_color_set(Evas_Object *obj, int r, int g, int b, int a)
    obj->cur.color.g = g;
    obj->cur.color.b = b;
    evas_object_clip_dirty(obj);
-   if ((obj->cur.color.a == 0) && (a == 0)) return;
+   if ((obj->cur.color.a == 0) && (a == 0) && (obj->cur.render_op == EVAS_RENDER_BLEND)) return;
    obj->cur.color.a = a;
    evas_object_change(obj);
 }
@@ -1300,9 +1288,9 @@ evas_object_anti_alias_set(Evas_Object *obj, Eina_Bool anti_alias)
    return;
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
-   if (obj->cur.anti_alias == !!anti_alias)
-     return;
-   obj->cur.anti_alias = !!anti_alias;
+   anti_alias = !!anti_alias;
+   if (obj->cur.anti_alias == anti_alias)return;
+   obj->cur.anti_alias = anti_alias;
    evas_object_change(obj);
 }
 
@@ -1337,8 +1325,7 @@ evas_object_scale_set(Evas_Object *obj, double scale)
    return;
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
-   if (obj->cur.scale == scale)
-     return;
+   if (obj->cur.scale == scale) return;
    obj->cur.scale = scale;
    evas_object_change(obj);
    if (obj->func->scale_update) obj->func->scale_update(obj);
@@ -1362,44 +1349,6 @@ evas_object_scale_get(const Evas_Object *obj)
    return obj->cur.scale;
 }
 
-/**
- * Sets the color_space to be used for linear interpolation of colors.
- *
- * @param   obj The given evas object.
- * @param   color_space one of EVAS_COLOR_SPACE_ARGB or EVAS_COLOR_SPACE_AHSV.
- *
- * @ingroup Evas_Object_Group_Extras
- */
-EAPI void
-evas_object_color_interpolation_set(Evas_Object *obj, int color_space)
-{
-   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
-   return;
-   MAGIC_CHECK_END();
-   if (obj->delete_me) return;
-   if (obj->cur.interpolation_color_space == color_space)
-	return;
-   obj->cur.interpolation_color_space = color_space;
-   evas_object_change(obj);
-}
-
-
-/**
- * Retrieves the current value of the color space used for linear interpolation.
- * @param   obj The given evas object.
- * @return  @c EVAS_COLOR_SPACE_ARGB or EVAS_COLOR_SPACE_AHSV.
- * @ingroup Evas_Object_Group_Extras
- */
-EAPI int
-evas_object_color_interpolation_get(const Evas_Object *obj)
-{
-   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
-   return 0;
-   MAGIC_CHECK_END();
-   if (obj->delete_me) return 0;
-   return obj->cur.interpolation_color_space;
-}
-
 /**
  * Sets the render_op to be used for rendering the evas object.
  * @param   obj The given evas object.
@@ -1414,12 +1363,11 @@ evas_object_render_op_set(Evas_Object *obj, Evas_Render_Op render_op)
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
    if ((Evas_Render_Op)obj->cur.render_op == render_op)
-	return;
+      return;
    obj->cur.render_op = render_op;
    evas_object_change(obj);
 }
 
-
 /**
  * Retrieves the current value of the operation used for rendering the evas object.
  * @param   obj The given evas object.
@@ -1708,3 +1656,42 @@ evas_object_precise_is_inside_get(const Evas_Object *obj)
    MAGIC_CHECK_END();
    return obj->precise_is_inside;
 }
+
+/**
+ * Set a hint flag on the object that this is used as a static "clipper".
+ * 
+ * This is a hint to evas that this object is used as a big static clipper
+ * and shouldnt be moved with children and otherwise considered specially. The
+ * Default is off.
+ * 
+ * @param obj The given object.
+ * @param is_static_clip The static clip flag (on or off)
+ * @ingroup Evas_Object_Group_Extras
+ */
+EAPI void
+evas_object_static_clip_set(Evas_Object *obj, Eina_Bool is_static_clip)
+{
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   obj->is_static_clip = is_static_clip;
+}
+
+/**
+ * Get static "clipper" hint flag.
+ * 
+ * @see evas_object_static_clip_set()
+ * 
+ * @param obj The given object.
+ * @return The static clip flag (on or off)
+ * @ingroup Evas_Object_Group_Extras
+ */
+EAPI Eina_Bool
+evas_object_static_clip_get(const Evas_Object *obj)
+{
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return 0;
+   MAGIC_CHECK_END();
+   return obj->is_static_clip;
+}
+
