@@ -2,7 +2,8 @@
 #include "evas_private.h"
 
 // debug rendering
-//#define REND_DGB 1
+/* #define REND_DGB 1 */
+/* #define STDOUT_DBG 1 */
 
 #ifdef REND_DGB
 static FILE *dbf = NULL;
@@ -12,7 +13,11 @@ rend_dbg(const char *txt)
 {
    if (!dbf)
      {
+#ifdef STDOUT_DBG
+        dbf = stdout;
+#else           
         dbf = fopen("EVAS-RENDER-DEBUG.log", "w");
+#endif        
         if (!dbf) return;
      }
    fputs(txt, dbf);
@@ -130,12 +135,14 @@ _evas_render_has_map(Evas_Object *obj)
 {
    return ((!((obj->func->can_map) && (obj->func->can_map(obj)))) &&
            ((obj->cur.map) && (obj->cur.usemap)));
+//   return ((obj->cur.map) && (obj->cur.usemap));
 }
 
 static Eina_Bool
 _evas_render_had_map(Evas_Object *obj)
 {
-   return ((!obj->cur.map) && (obj->prev.usemap));
+   return ((obj->prev.map) && (obj->prev.usemap));
+//   return ((!obj->cur.map) && (obj->prev.usemap));
 }
 
 static Eina_Bool
@@ -177,10 +184,10 @@ _evas_render_cur_clip_cache_del(Evas *e, Evas_Object *obj)
 }
 
 static void
-_evas_render_phase1_direct(Evas *e, 
-                           Eina_Array *active_objects, 
-                           Eina_Array *restack_objects, 
-                           Eina_Array *delete_objects, 
+_evas_render_phase1_direct(Evas *e,
+                           Eina_Array *active_objects __UNUSED__,
+                           Eina_Array *restack_objects __UNUSED__,
+                           Eina_Array *delete_objects __UNUSED__,
                            Eina_Array *render_objects)
 {
    unsigned int i;
@@ -198,15 +205,23 @@ _evas_render_phase1_direct(Evas *e,
              obj->func->render_pre(obj);
              if (obj->pre_render_done)
                {
-                  if ((obj->smart.smart) && _evas_render_has_map(obj))
+                  RD("      pre-render-done smart:%p|%p  [%p, %i] | [%p, %i] has_map:%i had_map:%i\n",
+                     obj->smart.smart, 
+                     evas_object_smart_members_get_direct(obj),
+                     obj->cur.map, obj->cur.usemap,
+                     obj->prev.map, obj->prev.usemap,
+                     _evas_render_has_map(obj),
+                     _evas_render_had_map(obj));
+                  if ((obj->smart.smart) && 
+                      (_evas_render_has_map(obj)))
                     {
-                       RD("    has map + smart\n");
+                       RD("      has map + smart\n");
                        _evas_render_prev_cur_clip_cache_add(e, obj);
                     }
                }
              else if (_evas_render_had_map(obj))
                {
-                  RD("    no pre-render done\n");
+                  RD("      no pre-render done\n");
                   _evas_render_prev_cur_clip_cache_add(e, obj);
                }
           }
@@ -232,7 +247,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
                                    Eina_Array *restack_objects,
                                    Eina_Array *delete_objects,
                                    Eina_Array *render_objects,
-                                   int restack, int map
+                                   int restack, int map,
+                                   int *redraw_all
 #ifdef REND_DGB
                                    , int level
 #endif
@@ -241,7 +257,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
    Eina_Bool clean_them = EINA_FALSE;
    Evas_Object *obj2;
    int is_active;
-
+   Eina_Bool hmap;
+   
    obj->rect_del = 0;
    obj->render_pre = 0;
 
@@ -274,7 +291,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
      }
 #endif
 
-   if (_evas_render_has_map(obj)) map = 1;
+   map = _evas_render_has_map(obj);
+   hmap = _evas_render_had_map(obj);
 
    if ((restack) && (!map))
      {
@@ -293,6 +311,10 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
         RD("      obj mapped\n");
         if (obj->changed)
           {
+             if (map != hmap)
+               {
+                  *redraw_all = 1;
+               }
              evas_object_clip_recalc(obj);
              if ((obj->restack) &&
                  (is_active) && (!obj->clip.clipees) &&
@@ -320,10 +342,23 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
         RD("      had map - restack objs\n");
 //        eina_array_push(restack_objects, obj);
         _evas_render_prev_cur_clip_cache_add(e, obj);
+        if (obj->changed)
+          {
+             if (hmap)
+               {
+                  if (!map)
+                    {
+                       if ((obj->cur.map) && (obj->cur.usemap)) map = 1;
+                    }
+               }
+             if (map != hmap)
+               {
+                  *redraw_all = 1;
+               }
+          }
      }
 
    /* handle normal rendering. this object knows how to handle maps */
-
    if (obj->changed)
      {
 	if (obj->smart.smart)
@@ -340,7 +375,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
 						     delete_objects,
 						     render_objects,
 						     obj->restack,
-                                                     map
+                                                     map,
+                                                     redraw_all
 #ifdef REND_DGB
                                                      , level + 1
 #endif
@@ -393,7 +429,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
 							  restack_objects,
 							  delete_objects,
 							  render_objects,
-							  restack, map
+							  restack, map,
+                                                          redraw_all
 #ifdef REND_DGB
                                                           , level + 1
 #endif
@@ -439,7 +476,8 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj,
                                                      restack_objects,
                                                      delete_objects,
                                                      render_objects,
-                                                     restack, map
+                                                     restack, map,
+                                                     redraw_all
 #ifdef REND_DGB
                                                      , level + 1
 #endif
@@ -459,7 +497,8 @@ _evas_render_phase1_process(Evas *e,
                             Eina_Array *active_objects,
                             Eina_Array *restack_objects,
                             Eina_Array *delete_objects,
-                            Eina_Array *render_objects)
+                            Eina_Array *render_objects,
+                            int *redraw_all)
 {
    Evas_Layer *lay;
    Eina_Bool clean_them = EINA_FALSE;
@@ -473,7 +512,7 @@ _evas_render_phase1_process(Evas *e,
 	  {
 	     clean_them |= _evas_render_phase1_object_process
                (e, obj, active_objects, restack_objects, delete_objects,
-                render_objects, 0, 0
+                render_objects, 0, 0, redraw_all
 #ifdef REND_DGB
                 , 1
 #endif
@@ -481,7 +520,6 @@ _evas_render_phase1_process(Evas *e,
 	  }
      }
    RD("  ---]\n");
-
    return clean_them;
 }
 
@@ -565,13 +603,60 @@ pending_change(void *data, void *gdata __UNUSED__)
    if (obj->delete_me) return EINA_FALSE;
    if (obj->pre_render_done)
      {
-   RD("  OBJ [%p] pending change %i -> 0, pre %i\n", obj, obj->changed, obj->pre_render_done);
-   obj->pre_render_done = 0;
+        RD("  OBJ [%p] pending change %i -> 0, pre %i\n", obj, obj->changed, obj->pre_render_done);
+        obj->pre_render_done = 0;
 //// FIXME: this wipes out changes
-   obj->changed = 0;
+        obj->changed = 0;
+        obj->changed_move_only = 0;
+        obj->changed_nomove = 0;
+        obj->changed_move = 0;
      }
    return obj->changed ? EINA_TRUE : EINA_FALSE;
 }
+/*
+static void
+unchange(Evas_Object *obj)
+{
+   Evas_Object *obj2;
+
+   if (!obj->changed) return;
+   obj->changed = 0;
+   obj->changed_move_only = 0;
+   obj->changed_nomove = 0;
+   obj->changed_move = 0;
+   EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj2)
+     {
+        unchange(obj2);
+     }
+}
+
+static int
+chlist(Evas_Object *obj, int i)
+{
+   Evas_Object *obj2;
+   int j;
+   int ret = 0;
+
+   if (!obj->changed) return 0;
+   for (j = 0; j < i; j++) printf(" ");
+   printf("ch2 %p %s %i [%i %i %ix%i] v %i/%i [r%i] %p\n", obj, 
+          obj->type, 
+          obj->changed_move_only,
+          obj->cur.geometry.x,
+          obj->cur.geometry.y,
+          obj->cur.geometry.w,
+          obj->cur.geometry.h,
+          obj->cur.visible,
+          obj->prev.visible,
+          obj->restack,
+          obj->clip.clipees);
+   EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj2)
+     {
+        if (obj2->changed)
+           ret |= chlist(obj2, i + 1);
+     }
+}
+*/
 
 static Eina_Bool
 evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
@@ -630,16 +715,28 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
              return clean_them;
           }
 
+        pts[0].px = obj->cur.map->persp.px << FP;
+        pts[0].py = obj->cur.map->persp.py << FP;
+        pts[0].foc = obj->cur.map->persp.foc << FP;
+        pts[0].z0 = obj->cur.map->persp.z0 << FP;
+        
         p = obj->cur.map->points;
         p_end = p + 4;
         pt = pts;
         for (; p < p_end; p++, pt++)
           {
-             pt->x = (p->x + off_x) << FP;
-             pt->y = (p->y + off_y) << FP;
-             pt->z = (p->z)         << FP;
+             pt->x = (p->x + (double)off_x) * FP1;
+             pt->y = (p->y + (double)off_y) * FP1;
+             pt->z = (p->z)                 * FP1;
+             pt->fx = p->px;
+             pt->fy = p->py;
+             pt->fz = p->z;
              pt->u = p->u * FP1;
              pt->v = p->v * FP1;
+             if      (pt->u < 0) pt->u = 0;
+             else if (pt->u > (sw * FP1)) pt->u = (sw * FP1);
+             if      (pt->v < 0) pt->v = 0;
+             else if (pt->v > (sh * FP1)) pt->v = (sh * FP1);
              pt->col = ARGB_JOIN(p->a, p->r, p->g, p->b);
           }
 
@@ -672,17 +769,34 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
         if (obj->smart.smart)
           {
              Evas_Object *obj2;
-
+             
              EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj2)
                {
+                  if (!evas_object_is_visible(obj2) &&
+                      !evas_object_was_visible(obj2))
+                    {
+                       obj2->changed = 0;
+                       obj2->changed_move_only = 0;
+                       obj2->changed_nomove = 0;
+                       obj2->changed_move = 0;
+                       continue;
+                    }
                   if (obj2->changed)
                     {
-                       obj2->changed  = 0;
+//                       chlist(obj2, 0);
                        changed = 1;
+                       obj2->changed = 0;
+                       obj2->changed_move_only = 0;
+                       obj2->changed_nomove = 0;
+                       obj2->changed_move = 0;
                        break;
                     }
                }
+//             unchange(obj);
              obj->changed = 0;
+             obj->changed_move_only = 0;
+             obj->changed_nomove = 0;
+             obj->changed_move = 0;
           }
         else
           {
@@ -690,12 +804,17 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                {
                   changed = 1;
                   obj->changed = 0;
+                  obj->changed_move_only = 0;
+                  obj->changed_nomove = 0;
+                  obj->changed_move = 0;
                }
           }
 
         // clear surface before re-render
         if ((changed) && (obj->cur.map->surface))
           {
+             int off_x2, off_y2;
+             
              RDI(level);
              RD("        children redraw\n");
              // FIXME: calculate "changes" within map surface and only clear
@@ -716,8 +835,8 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                   e->engine.func->context_free(e->engine.data.output, ctx);
                }
              ctx = e->engine.func->context_new(e->engine.data.output);
-             off_x = -obj->cur.geometry.x;
-             off_y = -obj->cur.geometry.y;
+             off_x2 = -obj->cur.geometry.x;
+             off_y2 = -obj->cur.geometry.y;
              if (obj->smart.smart)
                {
                   EINA_INLIST_FOREACH
@@ -725,7 +844,7 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                     {
                        clean_them |= evas_render_mapped(e, obj2, ctx,
 							obj->cur.map->surface,
-							off_x, off_y, 1
+							off_x2, off_y2, 1
 #ifdef REND_DGB
 							, level + 1
 #endif
@@ -739,14 +858,15 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                   w = obj->cur.map->surface_w;
                   h = obj->cur.map->surface_h;
                   RECTS_CLIP_TO_RECT(x, y, w, h,
-                                     obj->cur.geometry.x + off_x,
-                                     obj->cur.geometry.y + off_y,
+                                     obj->cur.geometry.x + off_x2,
+                                     obj->cur.geometry.y + off_y2,
                                      obj->cur.geometry.w,
                                      obj->cur.geometry.h);
+
                   e->engine.func->context_clip_set(e->engine.data.output,
                                                    ctx, x, y, w, h);
                   obj->func->render(obj, e->engine.data.output, ctx,
-                                    obj->cur.map->surface, off_x, off_y);
+                                    obj->cur.map->surface, off_x2, off_y2);
                }
              e->engine.func->context_free(e->engine.data.output, ctx);
              rendered = 1;
@@ -761,9 +881,62 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                (e->engine.data.output, obj->cur.map->surface,
                 0, 0, obj->cur.map->surface_w, obj->cur.map->surface_h);
           }
-        obj->layer->evas->engine.func->image_map4_draw
-          (e->engine.data.output, e->engine.data.context, surface,
-           obj->cur.map->surface, pts, obj->cur.map->smooth, 0);
+        e->engine.func->context_clip_unset(e->engine.data.output,
+                                           e->engine.data.context);
+        if (obj->cur.map->surface) 
+          {
+             if (obj->smart.smart)
+               {
+                  if (obj->cur.clipper)
+                    {
+                       int x, y, w, h;
+                       Evas_Object *tobj;
+                       
+                       obj->cur.cache.clip.dirty = 1;
+                       tobj = obj->cur.map_parent;
+                       obj->cur.map_parent = obj->cur.clipper->cur.map_parent;
+                       evas_object_clip_recalc(obj);
+                       obj->cur.map_parent = tobj;
+                       x = obj->cur.cache.clip.x;
+                       y = obj->cur.cache.clip.y;
+                       w = obj->cur.cache.clip.w;
+                       h = obj->cur.cache.clip.h;
+                       RECTS_CLIP_TO_RECT(x, y, w, h,
+                                          obj->cur.clipper->cur.cache.clip.x,
+                                          obj->cur.clipper->cur.cache.clip.y,
+                                          obj->cur.clipper->cur.cache.clip.w,
+                                          obj->cur.clipper->cur.cache.clip.h);
+                       e->engine.func->context_clip_set(e->engine.data.output,
+                                                        e->engine.data.context,
+                                                        x + off_x, y + off_y, w, h);
+                    }
+               }
+             else
+               {
+                  if (obj->cur.clipper)
+                    {
+                       int x, y, w, h;
+                       
+                       evas_object_clip_recalc(obj);
+                       x = obj->cur.cache.clip.x;
+                       y = obj->cur.cache.clip.y;
+                       w = obj->cur.cache.clip.w;
+                       h = obj->cur.cache.clip.h;
+                       RECTS_CLIP_TO_RECT(x, y, w, h,
+                              obj->cur.clipper->cur.cache.clip.x,
+                              obj->cur.clipper->cur.cache.clip.y,
+                              obj->cur.clipper->cur.cache.clip.w,
+                              obj->cur.clipper->cur.cache.clip.h);
+                       e->engine.func->context_clip_set(e->engine.data.output,
+                                                        e->engine.data.context,
+                                                        x + off_x, y + off_y, w, h);
+                    }
+               }
+          }
+        if (obj->cur.cache.clip.visible)
+           obj->layer->evas->engine.func->image_map4_draw
+           (e->engine.data.output, e->engine.data.context, surface,
+            obj->cur.map->surface, pts, obj->cur.map->smooth, 0);
         // FIXME: needs to cache these maps and
         // keep them only rendering updates
 //        obj->layer->evas->engine.func->image_map_surface_free
@@ -774,6 +947,26 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
      {
         if (mapped)
           {
+             if (obj->cur.clipper)
+               {
+                  int x, y, w, h;
+
+                  if (_evas_render_has_map(obj))
+                    evas_object_clip_recalc(obj);
+                  x = obj->cur.cache.clip.x;
+                  y = obj->cur.cache.clip.y;
+                  w = obj->cur.cache.clip.w;
+                  h = obj->cur.cache.clip.h;
+                  RECTS_CLIP_TO_RECT(x, y, w, h,
+                                     obj->cur.clipper->cur.cache.clip.x,
+                                     obj->cur.clipper->cur.cache.clip.y,
+                                     obj->cur.clipper->cur.cache.clip.w,
+                                     obj->cur.clipper->cur.cache.clip.h);
+                  e->engine.func->context_clip_set(e->engine.data.output,
+                                                   e->engine.data.context,
+                                                   x + off_x, y + off_y, w, h);
+               }
+
              RDI(level);
              RD("        draw child of mapped obj\n");
              ctx = e->engine.func->context_new(e->engine.data.output);
@@ -792,12 +985,88 @@ evas_render_mapped(Evas *e, Evas_Object *obj, void *context, void *surface,
                     }
                }
              else
-               obj->func->render(obj, e->engine.data.output, ctx,
-                                 surface, off_x, off_y);
+               {
+                  if (!obj->cur.map)
+                    {
+                       int x, y, w, h;
+
+                       x = obj->cur.cache.clip.x + off_x;
+                       y = obj->cur.cache.clip.y + off_y;
+                       w = obj->cur.cache.clip.w;
+                       h = obj->cur.cache.clip.h;
+
+                       if (obj->cur.clipper)
+                         {
+                            if (_evas_render_has_map(obj))
+                              evas_object_clip_recalc(obj);
+
+                            RD("        clipper: %i %i %ix%i\n",
+                               obj->cur.clipper->cur.cache.clip.x + off_x,
+                               obj->cur.clipper->cur.cache.clip.y + off_y,
+                               obj->cur.clipper->cur.cache.clip.w,
+                               obj->cur.clipper->cur.cache.clip.h);
+
+                            RECTS_CLIP_TO_RECT(x, y, w, h,
+                                               obj->cur.clipper->cur.cache.clip.x + off_x,
+                                               obj->cur.clipper->cur.cache.clip.y + off_y,
+                                               obj->cur.clipper->cur.cache.clip.w,
+                                               obj->cur.clipper->cur.cache.clip.h);
+                         }
+
+                       RDI(level);
+                       RD("        clip: %i %i %ix%i [%i %i %ix%i]\n",
+                          obj->cur.cache.clip.x + off_x,
+                          obj->cur.cache.clip.y + off_y, 
+                          obj->cur.cache.clip.w,
+                          obj->cur.cache.clip.h,
+                          obj->cur.geometry.x + off_x,
+                          obj->cur.geometry.y + off_y,
+                          obj->cur.geometry.w,
+                          obj->cur.geometry.h);
+                       e->engine.func->context_clip_set(e->engine.data.output,
+                                                        ctx, x, y, w, h);
+                    }
+                  else
+                    {
+                       RDI(level);
+                       RD("        noclip\n");
+                    }
+                  obj->func->render(obj, e->engine.data.output, ctx,
+                                    surface, off_x, off_y);
+/*                  
+                  obj->layer->evas->engine.func->context_color_set(e->engine.data.output,
+                                                                   ctx,
+                                                                   0, 30, 0, 30);
+                  obj->layer->evas->engine.func->rectangle_draw(e->engine.data.output,
+                                                                ctx,
+                                                                surface,
+                                                                0, 0, 9999, 9999);
+ */
+               }
              e->engine.func->context_free(e->engine.data.output, ctx);
           }
         else
           {
+             if (obj->cur.clipper)
+               {
+                  int x, y, w, h;
+
+                  if (_evas_render_has_map(obj))
+                    evas_object_clip_recalc(obj);
+                  x = obj->cur.cache.clip.x;
+                  y = obj->cur.cache.clip.y;
+                  w = obj->cur.cache.clip.w;
+                  h = obj->cur.cache.clip.h;
+                  RECTS_CLIP_TO_RECT(x, y, w, h,
+                                     obj->cur.clipper->cur.cache.clip.x,
+                                     obj->cur.clipper->cur.cache.clip.y,
+                                     obj->cur.clipper->cur.cache.clip.w,
+                                     obj->cur.clipper->cur.cache.clip.h);
+                  e->engine.func->context_clip_set(e->engine.data.output,
+                                                   e->engine.data.context,
+                                                   x + off_x, y + off_y, w, h);
+               }
+
              RDI(level);
              RD("        draw normal obj\n");
              obj->func->render(obj, e->engine.data.output, context, surface,
@@ -825,6 +1094,7 @@ evas_render_updates_internal(Evas *e,
    int cx, cy, cw, ch;
    unsigned int i, j;
    int haveup = 0;
+   int redraw_all = 0;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return NULL;
@@ -841,9 +1111,15 @@ evas_render_updates_internal(Evas *e,
 
    /* phase 1. add extra updates for changed objects */
    if (e->invalidate || e->render_objects.count <= 0)
-     clean_them = _evas_render_phase1_process(e, &e->active_objects, &e->restack_objects, &e->delete_objects, &e->render_objects);
+     clean_them = _evas_render_phase1_process(e,
+                                              &e->active_objects,
+                                              &e->restack_objects,
+                                              &e->delete_objects,
+                                              &e->render_objects,
+                                              &redraw_all);
 
-   _evas_render_phase1_direct(e, &e->active_objects, &e->restack_objects, &e->delete_objects, &e->render_objects);
+   _evas_render_phase1_direct(e, &e->active_objects, &e->restack_objects,
+                              &e->delete_objects, &e->render_objects);
 
    /* phase 2. force updates for restacks */
    for (i = 0; i < e->restack_objects.count; ++i)
@@ -858,29 +1134,35 @@ evas_render_updates_internal(Evas *e,
    /* phase 3. add exposes */
    EINA_LIST_FREE(e->damages, r)
      {
-	e->engine.func->output_redraws_rect_add(e->engine.data.output,
-					       r->x, r->y, r->w, r->h);
-	eina_rectangle_free(r);
+        e->engine.func->output_redraws_rect_add(e->engine.data.output,
+                                                r->x, r->y, r->w, r->h);
+        eina_rectangle_free(r);
      }
    /* phase 4. output & viewport changes */
    if (e->viewport.changed)
      {
-	e->engine.func->output_redraws_rect_add(e->engine.data.output,
-						0, 0,
-						e->output.w, e->output.h);
+        e->engine.func->output_redraws_rect_add(e->engine.data.output,
+                                                0, 0,
+                                                e->output.w, e->output.h);
      }
    if (e->output.changed)
      {
-	e->engine.func->output_resize(e->engine.data.output,
-				      e->output.w, e->output.h);
-	e->engine.func->output_redraws_rect_add(e->engine.data.output,
-						0, 0,
-						e->output.w, e->output.h);
+        e->engine.func->output_resize(e->engine.data.output,
+                                      e->output.w, e->output.h);
+        e->engine.func->output_redraws_rect_add(e->engine.data.output,
+                                                0, 0,
+                                                e->output.w, e->output.h);
      }
    if ((e->output.w != e->viewport.w) || (e->output.h != e->viewport.h))
      {
 	ERR("viewport size != output size!");
      }
+   if (redraw_all)
+     {
+        e->engine.func->output_redraws_rect_add(e->engine.data.output,
+                                                0, 0, 
+                                                e->output.w, e->output.h);
+     }
    /* phase 5. add obscures */
    EINA_LIST_FOREACH(e->obscures, ll, r)
      {
@@ -913,12 +1195,14 @@ evas_render_updates_internal(Evas *e,
      {
 	unsigned int offset = 0;
 
-	alpha = e->engine.func->canvas_alpha_get(e->engine.data.output, e->engine.data.context);
+	alpha = e->engine.func->canvas_alpha_get(e->engine.data.output, 
+                                                 e->engine.data.context);
 
 	while ((surface =
-		e->engine.func->output_redraws_next_update_get(e->engine.data.output,
-							       &ux, &uy, &uw, &uh,
-							       &cx, &cy, &cw, &ch)))
+		e->engine.func->output_redraws_next_update_get
+                (e->engine.data.output,
+                    &ux, &uy, &uw, &uh,
+                    &cx, &cy, &cw, &ch)))
 	  {
 	     int off_x, off_y;
 
@@ -939,41 +1223,46 @@ evas_render_updates_internal(Evas *e,
 	       {
 		  Evas_Object *obj;
 
-		  obj = (Evas_Object *) eina_array_data_get(&e->obscuring_objects, i);
+		  obj = (Evas_Object *)eina_array_data_get
+                     (&e->obscuring_objects, i);
 		  if (evas_object_is_in_output_rect(obj, ux, uy, uw, uh))
 		    {
 		       eina_array_push(&e->temporary_objects, obj);
-
+                       
 		       /* reset the background of the area if needed (using cutout and engine alpha flag to help) */
-		       if (alpha)
-			 {
-			    if (evas_object_is_opaque(obj))
-			      e->engine.func->context_cutout_add(e->engine.data.output,
-								 e->engine.data.context,
-								 obj->cur.cache.clip.x + off_x,
-								 obj->cur.cache.clip.y + off_y,
-								 obj->cur.cache.clip.w,
-								 obj->cur.cache.clip.h);
+                       if (alpha)
+                         {
+                            if (evas_object_is_opaque(obj))
+                               e->engine.func->context_cutout_add
+                               (e->engine.data.output,
+                                   e->engine.data.context,
+                                   obj->cur.cache.clip.x + off_x,
+                                   obj->cur.cache.clip.y + off_y,
+                                   obj->cur.cache.clip.w,
+                                   obj->cur.cache.clip.h);
 			    else
 			      {
 				 if (obj->func->get_opaque_rect)
 				   {
 				      Evas_Coord obx, oby, obw, obh;
 
-				      obj->func->get_opaque_rect(obj, &obx, &oby, &obw, &obh);
+				      obj->func->get_opaque_rect
+                                         (obj, &obx, &oby, &obw, &obh);
 				      if ((obw > 0) && (obh > 0))
 					{
 					   obx += off_x;
 					   oby += off_y;
-					   RECTS_CLIP_TO_RECT(obx, oby, obw, obh,
-							      obj->cur.cache.clip.x + off_x,
-							      obj->cur.cache.clip.y + off_y,
-							      obj->cur.cache.clip.w,
-							      obj->cur.cache.clip.h);
-					   e->engine.func->context_cutout_add(e->engine.data.output,
-									      e->engine.data.context,
-									      obx, oby,
-									      obw, obh);
+					   RECTS_CLIP_TO_RECT
+                                              (obx, oby, obw, obh,
+                                                  obj->cur.cache.clip.x + off_x,
+                                                  obj->cur.cache.clip.y + off_y,
+                                                  obj->cur.cache.clip.w,
+                                                  obj->cur.cache.clip.h);
+					   e->engine.func->context_cutout_add
+                                              (e->engine.data.output,
+                                                  e->engine.data.context,
+                                                  obx, oby,
+                                                  obw, obh);
 					}
 				   }
 			      }
@@ -985,9 +1274,14 @@ evas_render_updates_internal(Evas *e,
 		  e->engine.func->context_clip_set(e->engine.data.output,
 						   e->engine.data.context,
 						   ux, uy, uw, uh);
-		  e->engine.func->context_color_set(e->engine.data.output, e->engine.data.context, 0, 0, 0, 0);
-		  e->engine.func->context_multiplier_unset(e->engine.data.output, e->engine.data.context);
-		  e->engine.func->context_render_op_set(e->engine.data.output, e->engine.data.context, EVAS_RENDER_COPY);
+		  e->engine.func->context_color_set(e->engine.data.output, 
+                                                    e->engine.data.context, 
+                                                    0, 0, 0, 0);
+		  e->engine.func->context_multiplier_unset
+                     (e->engine.data.output, e->engine.data.context);
+		  e->engine.func->context_render_op_set(e->engine.data.output, 
+                                                        e->engine.data.context,
+                                                        EVAS_RENDER_COPY);
 		  e->engine.func->rectangle_draw(e->engine.data.output,
 						 e->engine.data.context,
 						 surface,
@@ -1013,7 +1307,7 @@ evas_render_updates_internal(Evas *e,
 		      (!obj->delete_me) &&
 		      (obj->cur.cache.clip.visible) &&
 //		      (!obj->smart.smart) &&
-		      (obj->cur.color.a > 0))
+		      ((obj->cur.color.a > 0 || obj->cur.render_op != EVAS_RENDER_BLEND)))
 		    {
 		       int x, y, w, h;
 
@@ -1022,26 +1316,6 @@ evas_render_updates_internal(Evas *e,
 			   (eina_array_data_get(&e->temporary_objects, offset) == obj))
 			 offset++;
 		       x = cx; y = cy; w = cw; h = ch;
-                       if (obj->cur.clipper)
-                         {
-                            if (_evas_render_has_map(obj))
-                              {
-                                 evas_object_clip_recalc(obj);
-                              }
-/* hmmmm clip seems to kill eweather in elm-test
-                            printf("clip: %4i %4i %4ix%4i to %4i %4i %4ix%4i\n",
-                                   x, y, w, h,
-                                   obj->cur.cache.clip.x + off_x,
-                                   obj->cur.cache.clip.y + off_y,
-                                   obj->cur.cache.clip.w,
-                                   obj->cur.cache.clip.h);
- */
-                            RECTS_CLIP_TO_RECT(x, y, w, h,
-                                               obj->cur.cache.clip.x + off_x,
-                                               obj->cur.cache.clip.y + off_y,
-                                               obj->cur.cache.clip.w,
-                                               obj->cur.cache.clip.h);
-                         }
 		       if (((w > 0) && (h > 0)) || (obj->smart.smart))
 			 {
                             if (!obj->smart.smart)
@@ -1052,9 +1326,13 @@ evas_render_updates_internal(Evas *e,
                                                     obj->cur.cache.clip.w,
                                                     obj->cur.cache.clip.h);
                               }
-			    e->engine.func->context_clip_set(e->engine.data.output,
-							     e->engine.data.context,
-							     x, y, w, h);
+                            if (obj->cur.clipper)
+                               e->engine.func->context_clip_set(e->engine.data.output,
+                                                                e->engine.data.context,
+                                                                x, y, w, h);
+                            else
+                               e->engine.func->context_clip_unset(e->engine.data.output,
+                                                                  e->engine.data.context);
 #if 1 /* FIXME: this can slow things down... figure out optimum... coverage */
 			    for (j = offset; j < e->temporary_objects.count; ++j)
 			      {
@@ -1137,6 +1415,9 @@ evas_render_updates_internal(Evas *e,
 	     obj->func->render_post(obj);
 	     obj->restack = 0;
 	     obj->changed = 0;
+             obj->changed_move_only = 0;
+             obj->changed_nomove = 0;
+             obj->changed_move = 0;
 	  }
         else if ((obj->cur.map != obj->prev.map) ||
                  (obj->cur.usemap != obj->prev.usemap))
@@ -1145,6 +1426,9 @@ evas_render_updates_internal(Evas *e,
 	     obj->func->render_post(obj);
 	     obj->restack = 0;
 	     obj->changed = 0;
+             obj->changed_move_only = 0;
+             obj->changed_nomove = 0;
+             obj->changed_move = 0;
           }
 /* moved to other pre-process phase 1
 	if (obj->delete_me == 2)
@@ -1338,6 +1622,8 @@ evas_sync(Evas *e)
    MAGIC_CHECK_END();
 
    evas_common_frameq_flush ();
+#else
+   (void) e;
 #endif
 }
 
@@ -1388,7 +1674,11 @@ evas_render_dump(Evas *e)
         Evas_Object *obj;
 
         EINA_INLIST_FOREACH(lay->objects, obj)
-          _evas_render_dump_map_surfaces(obj);
+          {
+             if ((obj->type) && (!strcmp(obj->type, "image")))
+                evas_object_inform_call_image_unloaded(obj);
+             _evas_render_dump_map_surfaces(obj);
+          }
      }
    if ((e->engine.func) && (e->engine.func->output_dump) &&
        (e->engine.data.output))
@@ -1421,7 +1711,7 @@ evas_render_object_recalc(Evas_Object *obj)
 #ifndef EVAS_FRAME_QUEUING
    if ((!obj->changed) && (obj->delete_me < 2))
 #else
-   if ((!obj->changed) )
+   if ((!obj->changed))
 #endif
      {
 	Evas *e;
@@ -1429,7 +1719,7 @@ evas_render_object_recalc(Evas_Object *obj)
 	e = obj->layer->evas;
 	if ((!e) || (e->cleanup)) return;
 #ifdef EVAS_FRAME_QUEUING
-   if (obj->delete_me >= evas_common_frameq_get_frameq_sz() + 2) return;
+        if (obj->delete_me >= evas_common_frameq_get_frameq_sz() + 2) return;
 #endif
         eina_array_push(&e->pending_objects, obj);
 	obj->changed = 1;
