@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"  /* so that EAPI in Eet.h is correctly defined */
 #endif
@@ -37,13 +33,13 @@ static int                reference = 0;
 static Image_Entry      *_evas_common_rgba_image_new(void);
 static void              _evas_common_rgba_image_delete(Image_Entry *ie);
 
-static int               _evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h);
+static int               _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned int h);
 static void              _evas_common_rgba_image_surface_delete(Image_Entry *ie);
 static DATA32           *_evas_common_rgba_image_surface_pixels(Image_Entry *ie);
 
 static void              _evas_common_rgba_image_unload(Image_Entry *im);
 
-static void              _evas_common_rgba_image_dirty_region(Image_Entry *im, int x, int y, int w, int h);
+static void              _evas_common_rgba_image_dirty_region(Image_Entry *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 
 static int               _evas_common_rgba_image_ram_usage(Image_Entry *ie);
 
@@ -128,7 +124,7 @@ evas_common_image_shutdown(void)
 EAPI void
 evas_common_image_image_all_unload(void)
 {
-   evas_common_rgba_image_scalecache_flush();
+   evas_common_rgba_image_scalecache_dump();
    evas_cache_image_unload_all(eci);
 }
 
@@ -142,9 +138,9 @@ _evas_common_rgba_image_new(void)
    im->flags = RGBA_IMAGE_NOTHING;
    im->ref = 1;
 #ifdef EVAS_FRAME_QUEUING
-   LKI(im->ref_fq_add);
-   LKI(im->ref_fq_del);
-   pthread_cond_init(&(im->cond_fq_del), NULL);
+   LKI(im->cache_entry.ref_fq_add);
+   LKI(im->cache_entry.ref_fq_del);
+   pthread_cond_init(&(im->cache_entry.cond_fq_del), NULL);
 #endif
 
    evas_common_rgba_image_scalecache_init(&im->cache_entry);
@@ -159,9 +155,9 @@ _evas_common_rgba_image_delete(Image_Entry *ie)
 #ifdef BUILD_PIPE_RENDER
    evas_common_pipe_free(im);
 # ifdef EVAS_FRAME_QUEUING
-   LKD(im->ref_fq_add);
-   LKD(im->ref_fq_del);
-   pthread_cond_destroy(&(im->cond_fq_del));
+   LKD(im->cache_entry.ref_fq_add);
+   LKD(im->cache_entry.ref_fq_del);
+   pthread_cond_destroy(&(im->cache_entry.cond_fq_del));
 # endif
 #endif   
    evas_common_rgba_image_scalecache_shutdown(&im->cache_entry);
@@ -217,6 +213,10 @@ evas_common_rgba_image_unload(Image_Entry *ie)
         im->image.data = NULL;
         ie->allocated.w = 0;
         ie->allocated.h = 0;
+        ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+        ie->flags.preload_done = 0;
+#endif
         return;
      }
 #endif   
@@ -226,10 +226,14 @@ evas_common_rgba_image_unload(Image_Entry *ie)
    im->image.data = NULL;
    ie->allocated.w = 0;
    ie->allocated.h = 0;
+   ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+   ie->flags.preload_done = 0;
+#endif
 }
 
 static int
-_evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h)
+_evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned int h)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
    size_t        siz = 0;
@@ -246,7 +250,7 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h)
 
    if (im->image.data) free(im->image.data);
    im->image.data = malloc(siz);
-   if (im->image.data == NULL) return -1;
+   if (!im->image.data) return -1;
 
 #ifdef HAVE_VALGRIND
 # ifdef VALGRIND_MAKE_READABLE
@@ -267,7 +271,7 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
    RGBA_Image   *im = (RGBA_Image *) ie;
 
    if (ie->file)
-     printf("unload: [%p] %s %s\n", ie, ie->file, ie->key);
+     DBG("unload: [%p] %s %s", ie, ie->file, ie->key);
    if ((im->cs.data) && (im->image.data))
      {
 	if (im->cs.data != im->image.data)
@@ -288,18 +292,25 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
      evas_cserve_image_free(ie);
 #endif   
    im->image.data = NULL;
+   ie->allocated.w = 0;
+   ie->allocated.h = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+   ie->flags.preload_done = 0;
+#endif
+   ie->flags.loaded = 0;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
 }
 
 static void
 _evas_common_rgba_image_unload(Image_Entry *im)
 {
-//   printf("unload: [%p] %s %s\n", im, im->file, im->key);
+//   DBG("unload: [%p] %s %s", im, im->file, im->key);
+   evas_common_rgba_image_scalecache_dirty(im);
    evas_common_rgba_image_unload(im);
 }
 
 static void
-_evas_common_rgba_image_dirty_region(Image_Entry* ie, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
+_evas_common_rgba_image_dirty_region(Image_Entry* ie, unsigned int x __UNUSED__, unsigned int y __UNUSED__, unsigned int w __UNUSED__, unsigned int h __UNUSED__)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
 
@@ -447,7 +458,7 @@ evas_common_image_surface_alpha_tiles_calc(RGBA_Surface *is, int tsize)
 /* } */
 
 static RGBA_Image *
-evas_common_image_create(int w, int h)
+evas_common_image_create(unsigned int w, unsigned int h)
 {
    RGBA_Image *im;
 
@@ -465,7 +476,7 @@ evas_common_image_create(int w, int h)
 }
 
 EAPI RGBA_Image *
-evas_common_image_alpha_create(int w, int h)
+evas_common_image_alpha_create(unsigned int w, unsigned int h)
 {
    RGBA_Image   *im;
 
@@ -484,7 +495,7 @@ evas_common_image_alpha_create(int w, int h)
 }
 
 EAPI RGBA_Image *
-evas_common_image_new(int w, int h, int alpha)
+evas_common_image_new(unsigned int w, unsigned int h, unsigned int alpha)
 {
    if (alpha)
      return evas_common_image_alpha_create(w, h);
@@ -530,9 +541,9 @@ evas_common_image_colorspace_dirty(RGBA_Image *im)
 }
 
 EAPI void
-evas_common_image_set_cache(int size)
+evas_common_image_set_cache(unsigned int size)
 {
-   if (eci != NULL)
+   if (eci)
      evas_cache_image_set(eci, size);
 }
 
@@ -545,7 +556,7 @@ evas_common_image_get_cache(void)
 EAPI RGBA_Image *
 evas_common_load_image_from_file(const char *file, const char *key, RGBA_Image_Loadopts *lo, int *error)
 {
-   if (file == NULL)
+   if (!file)
      {
 	*error = EVAS_LOAD_ERROR_GENERIC;
 	return NULL;
