@@ -3,13 +3,14 @@
 #endif
 
 #include <stdio.h>
-#include <jpeglib.h>
-#include <setjmp.h>
 
 #ifdef HAVE_EVIL
 # include <Evil.h>
 #endif
 
+#include <setjmp.h>
+#include <jpeglib.h>
+
 #include "evas_common.h"
 #include "evas_private.h"
 
@@ -78,7 +79,7 @@ _JPEGErrorHandler2(j_common_ptr cinfo __UNUSED__, int msg_level __UNUSED__)
 static Eina_Bool
 evas_image_load_file_head_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
 {
-   int w, h, scalew, scaleh;
+   unsigned int w, h, scalew, scaleh;
    struct jpeg_decompress_struct cinfo;
    struct _JPEG_error_mgr jerr;
 
@@ -129,15 +130,18 @@ evas_image_load_file_head_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
      }
    else if ((ie->load_opts.w > 0) && (ie->load_opts.h > 0))
      {
-	int w2 = w, h2 = h;
+	unsigned int w2 = w, h2 = h;
 	if (ie->load_opts.w > 0)
 	  {
 	     w2 = ie->load_opts.w;
 	     h2 = (ie->load_opts.w * h) / w;
 	     if ((ie->load_opts.h > 0) && (h2 > ie->load_opts.h))
 	       {
+	          unsigned int w3;
 		  h2 = ie->load_opts.h;
-		  w2 = (ie->load_opts.h * w) / h;
+		  w3 = (ie->load_opts.h * w) / h;
+		  if (w3 > w2)
+		    w2 = w3;
 	       }
 	  }
 	else if (ie->load_opts.h > 0)
@@ -171,7 +175,6 @@ evas_image_load_file_head_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
    if (ie->scale > 1)
      {
 	jpeg_destroy_decompress(&cinfo);
-
 	rewind(f);
 	jpeg_create_decompress(&cinfo);
 	jpeg_stdio_src(&cinfo, f);
@@ -223,12 +226,12 @@ get_time(void)
 static Eina_Bool
 evas_image_load_file_data_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
 {
-   int w, h;
+   unsigned int w, h;
    struct jpeg_decompress_struct cinfo;
    struct _JPEG_error_mgr jerr;
    DATA8 *ptr, *line[16], *data;
    DATA32 *ptr2;
-   int x, y, l, i, scans, count;
+   unsigned int x, y, l, i, scans;
    int region = 0;
 
    cinfo.err = jpeg_std_error(&(jerr.pub));
@@ -331,12 +334,12 @@ evas_image_load_file_data_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
 	return EINA_TRUE;
      }
    ptr2 = evas_cache_image_pixels(ie);
-   count = 0;
+
    /* We handle first CMYK (4 components) */
    if (cinfo.output_components == 4)
      {
         // FIXME: handle region
-	for (i = 0; i < cinfo.rec_outbuf_height; i++)
+	for (i = 0; (int)i < cinfo.rec_outbuf_height; i++)
 	  line[i] = data + (i * w * 4);
 	for (l = 0; l < h; l += cinfo.rec_outbuf_height)
 	  {
@@ -479,7 +482,7 @@ evas_image_load_file_data_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
           }
         t = get_time();
  */
-        for (i = 0; i < cinfo.rec_outbuf_height; i++)
+        for (i = 0; (int)i < cinfo.rec_outbuf_height; i++)
 	  line[i] = data + (i * w * 3);
 	for (l = 0; l < h; l += cinfo.rec_outbuf_height)
 	  {
@@ -544,7 +547,7 @@ evas_image_load_file_data_jpeg_internal(Image_Entry *ie, FILE *f, int *error)
    /* We finally handle RGB with 1 component */
    else if (cinfo.output_components == 1)
      {
-	for (i = 0; i < cinfo.rec_outbuf_height; i++)
+	for (i = 0; (int)i < cinfo.rec_outbuf_height; i++)
 	  line[i] = data + (i * w);
 	for (l = 0; l < h; l += cinfo.rec_outbuf_height)
 	  {
@@ -614,7 +617,7 @@ evas_image_load_file_data_jpeg_alpha_internal(Image_Entry *ie, FILE *f, int *err
    struct _JPEG_error_mgr jerr;
    DATA8 *ptr, *line[16], *data;
    DATA32 *ptr2;
-   int x, y, l, i, scans, count, prevy;
+   int x, y, l, i, scans, prevy;
 
    if (!f)
      {
@@ -657,7 +660,6 @@ evas_image_load_file_data_jpeg_alpha_internal(Image_Entry *ie, FILE *f, int *err
 	return EINA_TRUE;
      }
    ptr2 = evas_cache_image_pixels(ie);
-   count = 0;
    prevy = 0;
    if (cinfo.output_components == 3)
      {
@@ -758,7 +760,7 @@ module_open(Evas_Module *em)
 }
 
 static void
-module_close(Evas_Module *em)
+module_close(Evas_Module *em __UNUSED__)
 {
 }
 
